<?xml version="1.0" encoding="UTF-8"?>
 
<#-- Import common macros and functions -->
<#import "macros_common_general.ftl" as com>
<#import "common_module_traversal_utils.ftl" as utils>
<#import "common_module_front_page_docbook.ftl" as fp>

<#assign locale = "en" />
<#assign sysDateTime = .now>

<#-- Initialize common variables -->
<@com.initializeMainVariables/>	

<#assign locale = "en" />
<#assign sysDateTime = .now>

<#-- Root entity -->
<#assign rootEntity = entity.root>

<#-- Define the main macro and the arguments to be used to get output using the traversal approach -->
<#assign mainMacroName = "addLiteratureReferencesForDocument"/>
<#assign mainMacroArgs = ['sectionDoc', 'sectionNode', 'entityDocument', 'level']/>

<#-- Make global the macroCall as well as each macro used in it -->
<#global mainMacroCall="<@" + mainMacroName + " " + mainMacroArgs?join(" ") + "/>"/>
<#global addLiteratureReferencesForDocument=addLiteratureReferencesForDocument/>

<#-- Initialize list to store literature references information (input to table) -->
<#assign litRefsList  = []/>

<book version="5.0" xmlns="http://docbook.org/ns/docbook" xmlns:xi="http://www.w3.org/2001/XInclude">

	<#-- Get front page -->
	<@fp.getFrontPage _subject _dossierHeader "Document L (List of Literature References) - sorted by Section"/>

	<#-- Chapters -->
	<chapter label="1">
		<title role="HEAD-1">List of Literature References (sorted by Section)</title>

		<#-- Do the traverse -->
		<@utils.traversal rootEntity 8/>
		
		<#-- Print output -->
		<@tableOutput litRefsList/>

	</chapter>
</book>
		
<#----------------------------------------------->
<#-- Macro and function definitions start here -->
<#----------------------------------------------->
<#macro addLiteratureReferencesForDocument sectionDoc sectionNode entityDocument level>

	<#if level==0>

		<#-- get the list of references: it's a list of lists, where the first element is the literature
		reference and the second the data protection claimed, which can be a picklist value, "N/A", or empty
		-->
  		<#assign listOfReferences = getReferencedLitRefEntitiesFromDoc(sectionDoc) >
    	<#if listOfReferences?has_content>
      		<#list listOfReferences as literatureReferenceDpList>
				<#local literatureReference=literatureReferenceDpList[0]/>
				<#local dataPr=literatureReferenceDpList[1]/>

				<#if literatureReference?has_content>

					<#-- Get the section name: Section + Endpoint and sort it -->
					<#local sectionName = "" />

					<#local sectionCode= '0' />
  					<#local letters = ['0', 'a', 'b', 'c', 'd', 'e', 'f', 'g', 'h', 'k', 'l', 'm', 'n', 'o', 'p', 'q', 'r', 's', 't', 'u', 'v', 'w', 'x', 'y', 'z']>
  

					<#if sectionNode?has_content>

						<#if sectionNode.number?has_content>
							<#local sectionName = sectionNode.number + " ">
		
							<#local sectionNumberList = sectionNode.number?split("-")/>
							<#local sectionNumberList = sectionNumberList[0]?split(".")/>
							
							<#list 1..5 as index>
											
								<#attempt>
									<#local sectionNumber =  sectionNumberList[index-1]/>
									<#local letter = letters[sectionNumber?number]/>
								<#recover>
									<#local letter = letters[0]/>
								</#attempt>

								<#local sectionCode = sectionCode+letter/>
								
							</#list>

						</#if>

						<#local sectionName = sectionName + sectionNode.title />

					</#if>

					<#-- Endpoint: add to section name -->
					<#if sectionDoc.hasElement("AdministrativeData.Endpoint") && sectionDoc.AdministrativeData.Endpoint?has_content>
						<#local endpoint><@com.picklist picklistValue=sectionDoc.AdministrativeData.Endpoint printDescription=false printRemarks=false/></#local>
					<#else>
						<#local endpoint="NA"/>
					</#if>

					<#-- Sorting for code -->
  					<#local sectionCode = sectionCode + '-' + endpoint + '-' + entityDocument.documentType + '_' + entityDocument.name/>

					<#-- Add to hash -->
					<#assign litRefsList = litRefsList + [{'ref': literatureReference, 'doc': sectionDoc, 'sect': sectionName, 'no': sectionCode, 'endpoint':endpoint, 'ent':entityDocument, 'dp':dataPr}]/>

        		</#if>
      		</#list>
    	</#if>
    </#if>
</#macro>

<#-- This function retrieves all the literature references from a document -->
<#function getReferencedLitRefEntitiesFromDoc docElementNode>
	<#local listOfLiteratureReferences = [] />

	<#if docElementNode?node_type == "document_reference">
		<#local listOfLiteratureReferences = checkAndAddLiteraturReferenceWithDP(docElementNode, docElementNode, listOfLiteratureReferences) />
		<#return listOfLiteratureReferences />
	</#if>

	<#if docElementNode?node_type == "document_references">
		<#list docElementNode as item>
			<#local listOfLiteratureReferences = checkAndAddLiteraturReferenceWithDP(item, docElementNode, listOfLiteratureReferences) />
		</#list>
		<#return listOfLiteratureReferences />
	</#if>

	<#if docElementNode?children?has_content>
		<#list docElementNode?children as child>
			<#local listOfLiteratureReferences = listOfLiteratureReferences + getReferencedLitRefEntitiesFromDoc(child)  />
		</#list>
	</#if>
	<#return listOfLiteratureReferences/>
</#function>


<#-- This function is a new version of the one above, in order to include both the lit reference and data protection claim -->
<#function checkAndAddLiteraturReferenceWithDP docKey docNode listOfReferencedEntities>
	<#local doc = iuclid.getDocumentForKey(docKey) />
	<#if doc?has_content && doc.documentType == "LITERATURE">
		<#list listOfReferencedEntities as docDpList>
			<#local document=docDpList[0]/>
			<#if document.documentKey == doc.documentKey>
				<#return listOfReferencedEntities>
			</#if>
		</#list>

		<#-- get DP -->
		<#local dataProt=""/>
		<#local docKeyParents=docNode?ancestors/>
		<#if docKeyParents?has_content>
			<#local docKeyParent = docKeyParents[0]/>
			<#if docKeyParent.hasElement("DataProtectionClaimed")>
				<#local dataProt=docKeyParent.DataProtectionClaimed/>
			<#else>
				<#local dataProt="N/A"/>
			</#if>
		</#if>

		<#-- add -->
		<#local newLitEntry = [doc, dataProt]/>
		<#return listOfReferencedEntities + [newLitEntry]>
	</#if>
	<#return listOfReferencedEntities>
</#function>


<#-- Table format for the literature references -->
<#macro tableOutput refList>
	<#-- final table output -->
	<para role="small">
	<table border="1">
        <title>Table of Literature References</title>
        
        <#--  with Study name and no vertebrate studies -->
         
          <col width="14%" />
          <col width="8%" />
          <col width="4%" />
          <col width="28%" />
          <col width="6%"/>
          <col width="12%" />
          <col width="6%" />
          <col width="11%" />
          <col width="11%" />	
          		
          <tbody>
			<tr valign='middle' align='center'>			       			
              <th colspan="1"><?dbfo bgcolor="#d6eaf8" ?><emphasis role="bold">IUCLID section (endpoint)</emphasis></th>
              <#-- <th colspan="1"><?dbfo bgcolor="#d6eaf8" ?><emphasis role="bold">IUCLID document name and entity</emphasis></th>		 -->		
              <th colspan="1"><?dbfo bgcolor="#d6eaf8" ?><emphasis role="bold">Author(s)</emphasis></th>
              <th colspan="1"><?dbfo bgcolor="#d6eaf8" ?><emphasis role="bold">Year</emphasis></th>
              <th colspan="1"><?dbfo bgcolor="#d6eaf8" ?><emphasis role="bold">Title<?linebreak?>
              																   Report No.<?linebreak?>
              																   Document No.<?linebreak?>
              																   Source, where different from company<?linebreak?>
              																   GLP<?linebreak?>
              																   Published or not</emphasis></th>
              <th colspan="1"><?dbfo bgcolor="#d6eaf8" ?><emphasis role="bold">Vert. study Y/N</emphasis></th>
              <th colspan="1"><?dbfo bgcolor="#d6eaf8" ?><emphasis role="bold">Owner</emphasis></th>
              <th colspan="1"><?dbfo bgcolor="#d6eaf8" ?><emphasis role="bold">Data protection Y/N</emphasis></th>	
              <th colspan="1"><?dbfo bgcolor="#d6eaf8" ?><emphasis role="bold">Justification if data protection is claimed</emphasis></th>	
              <th colspan="1"><?dbfo bgcolor="#d6eaf8" ?><emphasis role="bold">Previously used<?linebreak?> Y/N</emphasis></th>
            </tr>
            
			<#if refList?has_content>
			
				<#list refList?sort_by("no") as refEntry>

				
					<#local literatureReference=refEntry['ref']>
					<#local sectionDoc=refEntry['doc']/>
					<#local entityDoc=refEntry['ent']/>
							
					<#-- variables to extract field labels -->
					<@iuclid.label for=literatureReference.GeneralInfo.Name var="title"/>
					<@iuclid.label for=literatureReference.GeneralInfo.ReportNo var="reportNo"/>
					<@iuclid.label for=literatureReference.GeneralInfo.CompanyOwnerStudyNo var="studyNo"/>
					<@iuclid.label for=literatureReference.GeneralInfo.Source var="source"/>
					<@iuclid.label for=literatureReference.GeneralInfo.CompanyOwner var="owner"/>
					<@iuclid.label for=literatureReference.GeneralInfo.TestLab var="testingFacility"/>
					<@iuclid.label for=literatureReference.GeneralInfo.LiteratureType var="refType"/>


					<#local refUrl=utils.getDocumentUrl(literatureReference) />
					<#local docUrl=utils.getDocumentUrl(sectionDoc) />
					
					 <tr>      
					 	
						<#-- IUCLID Section name and number -->
					    <td>
					    	<ulink url="${docUrl}">
						      <@com.text refEntry['sect'] />
						      <#if refEntry['endpoint']?has_content && refEntry['endpoint']!="NA">
						      	<?linebreak?>(${refEntry['endpoint']})
						      </#if>
						      <#-- ${refEntry['no']} -->
						     </ulink>
					    </td>
			 			
					    <#-- Author -->									
					    <td>
					      <#if literatureReference.GeneralInfo.Author?has_content>
					      	<@com.text literatureReference.GeneralInfo.Author />	
					      <#else>
					          <emphasis>No author provided</emphasis>
					      </#if>
					    </td>
					  
					    <#-- Year -->				
					    <td>
					      <#if literatureReference.GeneralInfo.ReferenceYear?has_content>
					      <@com.number literatureReference.GeneralInfo.ReferenceYear />		
					        <#else>
					          <emphasis>No year provided</emphasis>
					      </#if>
					    </td>
					    
					    <#-- Title, report no, source, type -->
					    <#local sourceVal = literatureReference.GeneralInfo.Source />
					    <#local ownerVal = literatureReference.GeneralInfo.CompanyOwner />
					    <#local testingFacilityVal =literatureReference.GeneralInfo.TestLab/>

					    <td>
					      <ulink url="${refUrl}">
					      	<#if literatureReference.GeneralInfo.Name?has_content>
					          <@com.text literatureReference.GeneralInfo.Name/> 	
					        <#else>
					        <emphasis><@com.text title/> not provided</emphasis>
					      	</#if>
					      </ulink>
					      					      
					      <para>
						      <#if literatureReference.GeneralInfo.ReportNo?has_content>
						          <#-- <@com.text reportNo/>:  -->
						          <@com.text literatureReference.GeneralInfo.ReportNo />
						      <#else>
						          <emphasis>Report No. not provided</emphasis>
						      </#if>
						  </para>
					      
					      <para>
						      <#if literatureReference.GeneralInfo.CompanyOwnerStudyNo?has_content>
						          <#--  <@com.text studyNo/>: -->
						          <@com.text literatureReference.GeneralInfo.CompanyOwnerStudyNo />
						        <#else>
						          <emphasis>Document No. not provided</emphasis>
						      </#if>
					      </para>
					      
						  <para>
							  <#if sourceVal?has_content || testingFacilityVal?has_content>
									<#if sourceVal!=ownerVal && sourceVal!=testingFacilityVal>
							        	<@com.text sourceVal/>	
							        </#if>

							        <#if testingFacilityVal!=ownerVal>
							        	<@com.text testingFacilityVal/>	
							        </#if>
						      <#else>
						      		<emphasis>Source not provided</emphasis>  
							  </#if>
						  </para>
						  
						  <para>
							  <#if sectionDoc.hasElement("MaterialsAndMethods.GLPComplianceStatement") && sectionDoc.MaterialsAndMethods.GLPComplianceStatement?has_content>
							      GLP: <@com.picklist picklistValue=sectionDoc.MaterialsAndMethods.GLPComplianceStatement printDescription=false printRemarks=false />
							  <#else>
							      <emphasis>GLP information not available</emphasis>
							  </#if>
						  </para>
					      
					      <para>
						      <#if refType?has_content>
						      	<#local litRefTypeVal><@com.picklist literatureReference.GeneralInfo.LiteratureType /></#local>
						      	
						      	<#if litRefTypeVal=="publication" || litRefTypeVal=="publication (copyright not owned for reproduction)" || litRefTypeVal=="review article or handbook">
						      		Published
						      	<#else>
						      		<#if sourceVal?has_content>
						      			Published
						      		<#else>
						      			Unpublished
						      		</#if>
						      	</#if>
						      	(${litRefTypeVal})
						      	<#-- 
						      	<#if sectionDoc.hasElement("DataSource.DataAccess") && sectionDoc.DataSource.DataAccess?has_content>
						      		(<@com.picklist picklistValue=sectionDoc.DataSource.DataAccess printDescription=false printRemarks=false />)
						      	</#if>
						      	-->
						      <#else>
						          <emphasis><@com.text refType/> not provided</emphasis>
						      </#if>
					      	</para>
					    </td>
					    
			 			<#-- Vertebrate study -->
			 			<td>
			 				<#-- #OPTION1: leave blank. #OPTION2: Y for Tox, Ecotox, and residues () -->
			 			</td>
			 			
			 			
					    <#-- Study sponsor -->				
					     <td>
					     	<#if ownerVal?has_content>
					          <@com.text ownerVal/> 	
					        <#else>
					          <emphasis>Owner not provided</emphasis>
					      	</#if>
					    </td>
					    
					    <#-- Data protection claimed yes or no -->
					    <td>
					      
					    	<#local dataProtection=refEntry['dp']/>
					    	<#if dataProtection?has_content && !(dataProtection?is_string) >
					    		<#local dpValue><@com.picklist picklistValue=dataProtection printDescription=false printRemarks=false/></#local>	
				      			<#local dpValue=dpValue?replace("yes", "Y")/>
				      			<#local dpValue=dpValue?replace(", ", " (")/>
								${dpValue?replace("share", "share)")}
							<#elseif dataProtection?has_content>
								${dataProtection}
							<#else>
								N
					    	</#if>
					    </td>
					    
					    <td>
					    	<#local dataProtection=refEntry['dp']/>
					    	<#if dataProtection?has_content && !(dataProtection?is_string) >
					    		<@com.text dataProtection.remarks/>
							<#else>
								${dataProtection}
							</#if>
					    </td>
					    
					    <#-- Previously used (Change Log) -->
					    <td>  
						    <#local changeLogStatus=getChangeLogStatus(sectionDoc, entityDoc)/>
							${changeLogStatus}
						</td>
					</tr>

				</#list>
			</#if>
		</tbody>
	</table>
	</para>
</#macro>


<#-- This function checks if the document exists in the change log and retrieves the status -->
<#function getChangeLogStatus study entity>
	<#local changeLogFlag="N"/>

	<#local changeLogEntryList=[]/>

	<#local changeLogs = iuclid.getSectionDocumentsForParentKey(entity.documentKey, "FLEXIBLE_RECORD", "ChangeLog") />
	<#list changeLogs as changeLog>
		<#list changeLog.ChangeLog.ChangeLogEntries as changeLogEntry>
			<#local changeLogDoc=iuclid.getDocumentForKey(changeLogEntry.LinkToDocument)/>

			<#if changeLogDoc?has_content>
				<#if study.documentKey.uuid==changeLogDoc.documentKey.uuid>
					<#local changeLogStatus><@com.picklist changeLogEntry.Status/></#local>
					<#if changeLogStatus?has_content>
						<#if !(changeLogStatus?starts_with("new"))>
							<#local changeLogFlag="Y"/>
						</#if>
						<#-- <#local changeLogStatusList = changeLogStatusList + [changeLogStatus]/> -->
						<#local changeLogEntryList = changeLogEntryList + [changeLogEntry]/>
					</#if>
				</#if>
			</#if>
		</#list>
	</#list>

	<#local changeLogStatusMessage>${changeLogFlag}
		<#if changeLogEntryList?has_content>
			<#list changeLogEntryList as clEntry>
				(<@com.picklist clEntry.Status/>)<#if clEntry.Remark?has_content> - <@com.text clEntry.Remark/></#if>
				<#if clEntry?has_next>|</#if>
			</#list>
		</#if>
	</#local>

	<#return changeLogStatusMessage>
</#function>